from __future__ import print_function
from functools import lru_cache, wraps
from os import getenv
from pkg_resources import resource_filename
from textwrap import dedent
import logging
import sys

__all__ = ["get_matlab", "ensure_spm"]
PATH_M = resource_filename(__name__, "")
log = logging.getLogger(__name__)


@lru_cache()
def get_matlab(name=None):
    try:
        from matlab import engine
    except ImportError:
        raise ImportError(
            dedent(
                """\
        Please install MATLAB and its Python module.
        See https://www.mathworks.com/help/matlab/matlab_external/\
install-the-matlab-engine-for-python.html
        or
        https://www.mathworks.com/help/matlab/matlab_external/\
install-matlab-engine-api-for-python-in-nondefault-locations.html
        It's likely you need to do:

        cd "MATLABROOT\\extern\\engines\\python"
        {exe} setup.py build --build-base="BUILDDIR" install

        - MATLABROOT: start MATLAB and type `matlabroot` in the command window
          to find the relevant directory for the above command.
        - Fill in any temporary directory name for BUILDDIR (e.g. /tmp/builddir).
        - If installation fails due to write permissions, try appending `--user`
          to the above command.
        """
            ).format(exe=sys.executable)
        )
    started = engine.find_matlab()
    if not started or (name and name not in started):
        notify = True
        log.debug("Starting MATLAB")
    eng = engine.connect_matlab(name=name or getenv("SPM12_MATLAB_ENGINE", None))
    if notify:
        log.debug("MATLAB started")
    log.debug("adding SPM (%s) to MATLAB path", PATH_M)
    eng.addpath(PATH_M, nargout=0)
    return eng


@wraps(get_matlab)
def ensure_spm(name=None):
    eng = get_matlab(name)
    if not eng.exist("spm_jobman"):
        raise ImportError(
            dedent(
                """\
            MATLAB cannot find SPM.
            Please follow installation instructions at
            https://en.wikibooks.org/wiki/SPM/Download
            """
            )
        )
    return eng



from contextlib import contextmanager
from os import path, getenv
from shutil import rmtree
from tempfile import mkdtemp
from textwrap import dedent

from amypad.imio import nii
import numpy as np
import pytest

from spm12 import regseg

HOME = getenv("DATA_ROOT", path.expanduser("~"))
DATA = path.join(HOME, "Ab_PET_mMR_test")
skip_no_data = pytest.mark.skipif(
    not path.exists(DATA),
    reason="""\
Cannot find Ab_PET_mMR_test in ${DATA_ROOT:-~} (%s).
Get it from https://zenodo.org/record/3877529
"""
    % HOME,
)
mri = path.join(DATA, "T1w_N4", "t1_S00113_17598013_N4bias_cut.nii.gz")
pet = path.join(
    DATA,
    "testing_reference",
    "Ab_PET_mMR_ref",
    "basic",
    "17598013_t-3000-3600sec_itr-4_suvr.nii.gz",
)
mri2pet = np.array(
    [
        [0.99990508, 0.00800995, 0.01121016, -0.68164088],
        [-0.00806219, 0.99995682, 0.00462244, -1.16235105],
        [-0.01117265, -0.00471238, 0.99992648, -1.02167229],
        [0.0, 0.0, 0.0, 1.0],
    ]
)


@contextmanager
def tmpdir(*args, **kwargs):
    d = mkdtemp(*args, **kwargs)
    yield d
    rmtree(d)


def assert_equal_arrays(x, y, nmse_tol=0):
    if nmse_tol:
        if ((x - y) ** 2).mean() / (y ** 2).mean() < nmse_tol:
            return
    elif (x == y).all():
        return
    raise ValueError(
        dedent(
            """\
        Unequal arrays:x != y. min/mean/max(std):
        x: {:.3g}/{:.3g}/{:.3g}({:.3g})
        y: {:.3g}/{:.3g}/{:.3g}({:.3g})
        """
        ).format(
            x.min(),
            x.mean(),
            x.max(),
            x.std(),
            y.min(),
            y.mean(),
            y.max(),
            y.std(),
        )
    )


@skip_no_data
def test_coreg():
    with tmpdir() as outpath:
        res = regseg.coreg_spm(pet, mri, outpath=outpath)
    assert_equal_arrays(res["affine"], mri2pet, 1e-4)


@skip_no_data
def test_resample():
    with tmpdir() as outpath:
        res = regseg.resample_spm(pet, mri, mri2pet, outpath=outpath)
        res = nii.getnii(res)
        ref = nii.getnii(pet)
        assert res.shape == ref.shape
        assert not np.isnan(res).all()
